﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace TidyCmd
{
    /// <summary>
    /// Wrapper class to open and write to a log file.
    /// 
    /// Author: Dennis Lang - 2014
    /// http://home.comcast.net/~lang.dennis/    
    /// 
    /// ========== License ==========
    /// This file is part of TidyCmd.
    ///
    /// TidyCmd is free software: you can redistribute it and/or modify
    /// it under the terms of the GNU General Public License as published by
    /// the Free Software Foundation, either version 3 of the License, or
    /// (at your option) any later version.
    ///
    /// TidyCmd is distributed in the hope that it will be useful,
    /// but WITHOUT ANY WARRANTY; without even the implied warranty of
    /// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    /// GNU General Public License for more details.
    ///
    /// The GNU General Public License: <http://www.gnu.org/licenses/>.
    /// </summary>
    public class Log
    {
        public static string FilePath {
            get { return sLogInfo.mFilePath; }
            set
            {
                sLogInfo.mFilePath = value;
                sLogInfo.mRegKey.SetValue("logFile", sLogInfo.mFilePath);
            }
        }

        public static bool ConsoleInfoOutput
        {
            get { return sLogInfo.mConsoleInfoOutput; }
            set { sLogInfo.mConsoleInfoOutput = value; }
        }
        public static bool ConsoleErrorOutput
        {
            get { return sLogInfo.mConsoleErrorOutput; }
            set { sLogInfo.mConsoleErrorOutput = value; }
        }

        class LogInfo
        {
            public string mFilePath;
            public bool mConsoleInfoOutput;
            public bool mConsoleErrorOutput;
            public TextWriter mLogWriter;
            public Microsoft.Win32.RegistryKey mRegKey;

            public LogInfo()
            {
                mRegKey = Microsoft.Win32.Registry.CurrentUser.CreateSubKey(@"Software\TidyNet");
                mFilePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData), "TidyNet.log");
                mFilePath = this.mRegKey.GetValue("logFile", mFilePath) as string;
                mConsoleInfoOutput = false;
                mConsoleErrorOutput = true;
            }
        }

        static LogInfo sLogInfo = new LogInfo();

#if false
        public Log()
        {
            string appName = Path.GetFileNameWithoutExtension(Environment.GetCommandLineArgs()[0]);
        }
#endif

        public static TextWriter Open()
        {
            if (sLogInfo.mLogWriter == null)
            {
                try
                {
                    FileStream fs = new FileStream(FilePath, FileMode.Append, FileAccess.Write, FileShare.ReadWrite);
                    sLogInfo.mLogWriter = new StreamWriter(fs);
                    // sLogInfo.mLogWriter = File.AppendText(FilePath);
                }
                catch (DirectoryNotFoundException )
                {
                    Directory.CreateDirectory(Path.GetDirectoryName(FilePath));
                }
                catch (Exception ex)
                {
                    Console.Error.WriteLine(ex.Message + " " + FilePath);
                }
            }

            return sLogInfo.mLogWriter;
        }

        public static void Close()
        {
            if (null != sLogInfo.mLogWriter)
                sLogInfo.mLogWriter.Close();
            sLogInfo.mLogWriter = null;
        }

        private static void WriteLine(string msg)
        {
            Open().WriteLine(msg);
            Open().Flush();
        }

        public static void WriteError(string msg)
        {
            string errMsg = DateTime.Now.ToString("s") + " " + msg;
            WriteLine(errMsg);
            if (ConsoleErrorOutput)
            {
                Console.ForegroundColor = ConsoleColor.Red;
                Console.WriteLine(errMsg);
                Console.ForegroundColor = ConsoleColor.White;
            }
        }

        public static void WriteInfo(string msg)
        {
            string infoMsg = DateTime.Now.ToString("s") + " " + msg;
            WriteLine(infoMsg);
            if (ConsoleInfoOutput)
            {
                // Console.ForegroundColor = ConsoleColor.White;
                Console.WriteLine(infoMsg);
            }
        }

        public static void WriteInfo2(string msg)
        {
            string infoMsg = DateTime.Now.ToString("s") + " " + msg;
            WriteLine(infoMsg);
            Console.WriteLine(infoMsg);
        }
    }
}
