﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace TidyCmd
{
    public enum TidyConfigCategory
    {
      eMarkup,          /**< Markup options: (X)HTML version, etc */
      eDiagnostics,     /**< Diagnostics */
      ePrettyPrint,     /**< Output layout */
      eEncoding,        /**< Character encodings */
      eMiscellaneous    /**< File handling, message format, etc. */
    }  


    // Option IDs Used to get/set option values.
    public enum TidyOptionId
    {
      eUnknownOption,   /**< Unknown option! */
      eIndentSpaces,    /**< Indentation n spaces */
      eWrapLen,         /**< Wrap margin */
      eTabSize,         /**< Expand tabs to n spaces */

      eCharEncoding,    /**< In/out character encoding */
      eInCharEncoding,  /**< Input character encoding (if different) */
      eOutCharEncoding, /**< Output character encoding (if different) */    
      eNewline,         /**< Output line ending (default to platform) */

      eDocType,         /**< See doctype property */
  //    eDocTypeStr,      /**< User specified doctype */

      eDuplicateAttrs,  /**< Keep first or last duplicate attribute */
      eAltText,         /**< Default text for alt attribute */
  
      /* obsolete */
      eSlideStyle,      /**< Style sheet for slides: not used for anything yet */

      eErrFile,         /**< File name to write errors to */
      eOutFile,         /**< File name to write markup to */
      eWriteBack,       /**< If true then output tidied markup */
      eShowMarkup,      /**< If false, normal output is suppressed */
      eShowInfo,        /**< If true, info-level messages are shown */
      eShowWarnings,    /**< However errors are always shown */
      eQuiet,           /**< No 'Parsing X', guessed DTD or summary */
      eSmartIndent,     
      eIndentContent,   /**< Indent content of appropriate tags */
                        /**< "auto" does text/block level content indentation */
      eCoerceEndTags,   /**< Coerce end tags from start tags where probably intended */
      eOmitOptionalTags,/**< Suppress optional start tags and end tags */
      eHideEndTags,     /**< Legacy name for TidyOmitOptionalTags */
      eXmlTags,         /**< Treat input as XML */
      eRawOut,          /**< no formatting, just fixing ? */
      eXmlOut,          /**< Create output as XML */
      eXhtmlOut,        /**< Output extensible HTML */
      eHtmlOut,         /**< Output plain HTML, even for XHTML input.
                             Yes means set explicitly. */
      eXmlDecl,         /**< Add <?xml?> for XML docs */
      eUpperCaseTags,   /**< Output tags in upper not lower case */
      eUpperCaseAttrs,  /**< Output attributes in upper not lower case */
      eMakeBare,        /**< Make bare HTML: remove Microsoft cruft */
      eMakeClean,       /**< Replace presentational clutter by style rules */
      eGDocClean,       /**< Clean up HTML exported from Google Docs */
      eLogicalEmphasis, /**< Replace i by em and b by strong */
      eDropPropAttrs,   /**< Discard proprietary attributes */
      eDropFontTags,    /**< Discard presentation tags */
      eDropEmptyElems,  /**< Discard empty elements */
      eDropEmptyParas,  /**< Discard empty p elements */
      eFixComments,     /**< Fix comments with adjacent hyphens */
      eBreakBeforeBR,   /**< Output newline before <br> or not? */

      /* obsolete */
      eBurstSlides,     /**< Create slides on each h2 element */

      eNumEntities,     /**< Use numeric entities */
      eQuoteMarks,      /**< Output " marks as &quot; */
      eQuoteNbsp,       /**< Output non-breaking space as entity */
      eQuoteAmpersand,  /**< Output naked ampersand as &amp; */
      eWrapAttVals,     /**< Wrap within attribute values */
      eWrapScriptlets,  /**< Wrap within JavaScript string literals */
      eWrapSection,     /**< Wrap within <![ ... ]> section tags */
      eWrapAsp,         /**< Wrap within ASP pseudo elements */
      eWrapJste,        /**< Wrap within JSTE pseudo elements */
      eWrapPhp,         /**< Wrap within PHP pseudo elements */
      eFixBackslash,    /**< Fix URLs by replacing \ with / */
      eIndentAttributes,/**< Newline+indent before each attribute */
      eXmlPIs,          /**< If set to yes PIs must end with ?> */
      eXmlSpace,        /**< If set to yes adds xml:space attr as needed */
      eEncloseBodyText, /**< If yes text at body is wrapped in P's */
      eEncloseBlockText,/**< If yes text in blocks is wrapped in P's */
      eKeepFileTimes,   /**< If yes last modied time is preserved */
      eWord2000,        /**< Draconian cleaning for Word2000 */
      eMark,            /**< Add meta element indicating tidied doc */
      eEmacs,           /**< If true format error output for GNU Emacs */
      eEmacsFile,       /**< Name of current Emacs file */
      eLiteralAttribs,  /**< If true attributes may use newlines */
      eBodyOnly,        /**< Output BODY content only */
      eFixUri,          /**< Applies URI encoding if necessary */
      eLowerLiterals,   /**< Folds known attribute values to lower case */
      eHideComments,    /**< Hides all (real) comments in output */
      eIndentCdata,     /**< Indent <!CDATA[ ... ]]> section */
      eForceOutput,     /**< Output document even if errors were found */
      eShowErrors,      /**< Number of errors to put out */
      eAsciiChars,      /**< Convert quotes and dashes to nearest ASCII char */
      eJoinClasses,     /**< Join multiple class attributes */
      eJoinStyles,      /**< Join multiple style attributes */
      eEscapeCdata,     /**< Replace <![CDATA[]]> sections with escaped text */

#if SUPPORT_ASIAN_ENCODINGS
      eLanguage,        /**< Language property: not used for anything yet */
      eNCR,             /**< Allow numeric character references */
#else
      eLanguageNotUsed,
      eNCRNotUsed,
#endif
#if SUPPORT_UTF16_ENCODINGS
      eOutputBOM,      /**< Output a Byte Order Mark (BOM) for UTF-16 encodings */
                        /**< auto: if input stream has BOM, we output a BOM */
#else
      eOutputBOMNotUsed,
#endif

      eReplaceColor,    /**< Replace hex color attribute values with names */
      eCSSPrefix,       /**< CSS class naming for -clean option */

      eInlineTags,      /**< Declared inline tags */
      eBlockTags,       /**< Declared block tags */
      eEmptyTags,       /**< Declared empty tags */
      ePreTags,         /**< Declared pre tags */

      eAccessibilityCheckLevel, /**< Accessibility check level 
                                       0 (old style), or 1, 2, 3 */

      eVertSpace,       /**< degree to which markup is spread out vertically */
#if SUPPORT_ASIAN_ENCODINGS
      ePunctWrap,       /**< consider punctuation and breaking spaces for wrapping */
#else
      ePunctWrapNotUsed,
#endif
      eMergeEmphasis,       /**< Merge nested B and I elements */
      eMergeDivs,           /**< Merge multiple DIVs */
      eDecorateInferredUL,  /**< Mark inferred UL elements with no indent CSS */
      ePreserveEntities,    /**< Preserve entities */
      eSortAttributes,      /**< Sort attributes */
      eMergeSpans,          /**< Merge multiple SPANs */
      eAnchorAsName,        /**< Define anchors as name attributes */
      eN_TIDY_OPTIONS       /**< Must be last */
    } 

    //  Option data types
    public enum TidyOptionType
    {
      eString,          /**< String */
      eInteger,         /**< Integer or enumeration */
      eBoolean          /**< Boolean flag */
    }

    // AutoBool values used by ParseBool, ParseTriState, ParseIndent, ParseBOM
    public enum TidyTriState
    {
       eNo,     /**< maps to 'no' */
       eYes,    /**< maps to 'yes' */
       eAuto    /**< Automatic */
    }

    // TidyNewline option values to control output line endings.
    public enum TidyLineEnding
    {
        eLF,         /**< Use Unix style: LF */
        eCRLF,       /**< Use DOS/Windows style: CR+LF */
        eCR          /**< Use Macintosh style: CR */
    }


    // Mode controlling treatment of doctype
    public enum TidyDoctypeModes
    {
        eHtml5,   /**< <!DOCTYPE html> */
        eOmit,    /**< Omit DOCTYPE altogether */
        eAuto,    /**< Keep DOCTYPE in input.  Set version to content */
        eStrict,  /**< Convert document to HTML 4 strict content model */
        eLoose,   /**< Convert document to HTML 4 transitional  content model */
        eUser     /**< Set DOCTYPE FPI explicitly */
    }

    // Mode controlling treatment of duplicate Attributes
    public enum TidyDupAttrModes
    {
        eKeepFirst,
        eKeepLast
    }

    // Mode controlling treatment of sorting attributes
    public enum TidyAttrSortStrategy
    {
        eNone,
        eAlpha
    }

/* I/O and Message handling interface
**
** By default, Tidy will define, create and use 
** instances of input and output handlers for 
** standard C buffered I/O (i.e. FILE* stdin,
** FILE* stdout and FILE* stderr for content
** input, content output and diagnostic output,
** respectively.  A FILE* cfgFile input handler
** will be used for config files.  Command line
** options will just be set directly.
*/

    // Message severity level
    public enum TidyReportLevel
    {
      eInfo,             /**< Information about markup usage */
      eWarning,          /**< Warning message */
      eConfig,           /**< Configuration error */
      eAccess,           /**< Accessibility message */
      eError,            /**< Error message - output suppressed */
      eBadDocument,      /**< I/O or file system error */
      eFatal             /**< Crash! */
    }


/* Document tree traversal functions
*/

    // Node types
    public enum TidyNodeType
    {
      eNode_Root,        /**< Root */
      eNode_DocType,     /**< DOCTYPE */
      eNode_Comment,     /**< Comment */
      eNode_ProcIns,     /**< Processing Instruction */
      eNode_Text,        /**< Text */
      eNode_Start,       /**< Start Tag */
      eNode_End,         /**< End Tag */
      eNode_StartEnd,    /**< Start/End (empty) Tag */
      eNode_CDATA,       /**< Unparsed Text */
      eNode_Section,     /**< XML Section */
      eNode_Asp,         /**< ASP Source */
      eNode_Jste,        /**< JSTE Source */
      eNode_Php,         /**< PHP Source */
      eNode_XmlDecl      /**< XML Declaration */
    }


    // Known HTML element types
    public enum TidyTagId
    {
      TidyTag_UNKNOWN,  /**< Unknown tag! */
      TidyTag_A,        /**< A */
      TidyTag_ABBR,     /**< ABBR */
      TidyTag_ACRONYM,  /**< ACRONYM */
      TidyTag_ADDRESS,  /**< ADDRESS */
      TidyTag_ALIGN,    /**< ALIGN */
      TidyTag_APPLET,   /**< APPLET */
      TidyTag_AREA,     /**< AREA */
      TidyTag_B,        /**< B */
      TidyTag_BASE,     /**< BASE */
      TidyTag_BASEFONT, /**< BASEFONT */
      TidyTag_BDO,      /**< BDO */
      TidyTag_BGSOUND,  /**< BGSOUND */
      TidyTag_BIG,      /**< BIG */
      TidyTag_BLINK,    /**< BLINK */
      TidyTag_BLOCKQUOTE,   /**< BLOCKQUOTE */
      TidyTag_BODY,     /**< BODY */
      TidyTag_BR,       /**< BR */
      TidyTag_BUTTON,   /**< BUTTON */
      TidyTag_CAPTION,  /**< CAPTION */
      TidyTag_CENTER,   /**< CENTER */
      TidyTag_CITE,     /**< CITE */
      TidyTag_CODE,     /**< CODE */
      TidyTag_COL,      /**< COL */
      TidyTag_COLGROUP, /**< COLGROUP */
      TidyTag_COMMENT,  /**< COMMENT */
      TidyTag_DD,       /**< DD */
      TidyTag_DEL,      /**< DEL */
      TidyTag_DFN,      /**< DFN */
      TidyTag_DIR,      /**< DIR */
      TidyTag_DIV,      /**< DIF */
      TidyTag_DL,       /**< DL */
      TidyTag_DT,       /**< DT */
      TidyTag_EM,       /**< EM */
      TidyTag_EMBED,    /**< EMBED */
      TidyTag_FIELDSET, /**< FIELDSET */
      TidyTag_FONT,     /**< FONT */
      TidyTag_FORM,     /**< FORM */
      TidyTag_FRAME,    /**< FRAME */
      TidyTag_FRAMESET, /**< FRAMESET */
      TidyTag_H1,       /**< H1 */
      TidyTag_H2,       /**< H2 */
      TidyTag_H3,       /**< H3 */
      TidyTag_H4,       /**< H4 */
      TidyTag_H5,       /**< H5 */
      TidyTag_H6,       /**< H6 */
      TidyTag_HEAD,     /**< HEAD */
      TidyTag_HR,       /**< HR */
      TidyTag_HTML,     /**< HTML */
      TidyTag_I,        /**< I */
      TidyTag_IFRAME,   /**< IFRAME */
      TidyTag_ILAYER,   /**< ILAYER */
      TidyTag_IMG,      /**< IMG */
      TidyTag_INPUT,    /**< INPUT */
      TidyTag_INS,      /**< INS */
      TidyTag_ISINDEX,  /**< ISINDEX */
      TidyTag_KBD,      /**< KBD */
      TidyTag_KEYGEN,   /**< KEYGEN */
      TidyTag_LABEL,    /**< LABEL */
      TidyTag_LAYER,    /**< LAYER */
      TidyTag_LEGEND,   /**< LEGEND */
      TidyTag_LI,       /**< LI */
      TidyTag_LINK,     /**< LINK */
      TidyTag_LISTING,  /**< LISTING */
      TidyTag_MAP,      /**< MAP */
      TidyTag_MARQUEE,  /**< MARQUEE */
      TidyTag_MENU,     /**< MENU */
      TidyTag_META,     /**< META */
      TidyTag_MULTICOL, /**< MULTICOL */
      TidyTag_NOBR,     /**< NOBR */
      TidyTag_NOEMBED,  /**< NOEMBED */
      TidyTag_NOFRAMES, /**< NOFRAMES */
      TidyTag_NOLAYER,  /**< NOLAYER */
      TidyTag_NOSAVE,   /**< NOSAVE */
      TidyTag_NOSCRIPT, /**< NOSCRIPT */
      TidyTag_OBJECT,   /**< OBJECT */
      TidyTag_OL,       /**< OL */
      TidyTag_OPTGROUP, /**< OPTGROUP */
      TidyTag_OPTION,   /**< OPTION */
      TidyTag_P,        /**< P */
      TidyTag_PARAM,    /**< PARAM */
      TidyTag_PLAINTEXT,/**< PLAINTEXT */
      TidyTag_PRE,      /**< PRE */
      TidyTag_Q,        /**< Q */
      TidyTag_RB,       /**< RB */
      TidyTag_RBC,      /**< RBC */
      TidyTag_RP,       /**< RP */
      TidyTag_RT,       /**< RT */
      TidyTag_RTC,      /**< RTC */
      TidyTag_RUBY,     /**< RUBY */
      TidyTag_S,        /**< S */
      TidyTag_SAMP,     /**< SAMP */
      TidyTag_SCRIPT,   /**< SCRIPT */
      TidyTag_SELECT,   /**< SELECT */
      TidyTag_SERVER,   /**< SERVER */
      TidyTag_SERVLET,  /**< SERVLET */
      TidyTag_SMALL,    /**< SMALL */
      TidyTag_SPACER,   /**< SPACER */
      TidyTag_SPAN,     /**< SPAN */
      TidyTag_STRIKE,   /**< STRIKE */
      TidyTag_STRONG,   /**< STRONG */
      TidyTag_STYLE,    /**< STYLE */
      TidyTag_SUB,      /**< SUB */
      TidyTag_SUP,      /**< SUP */
      TidyTag_TABLE,    /**< TABLE */
      TidyTag_TBODY,    /**< TBODY */
      TidyTag_TD,       /**< TD */
      TidyTag_TEXTAREA, /**< TEXTAREA */
      TidyTag_TFOOT,    /**< TFOOT */
      TidyTag_TH,       /**< TH */
      TidyTag_THEAD,    /**< THEAD */
      TidyTag_TITLE,    /**< TITLE */
      TidyTag_TR,       /**< TR */
      TidyTag_TT,       /**< TT */
      TidyTag_U,        /**< U */
      TidyTag_UL,       /**< UL */
      TidyTag_VAR,      /**< VAR */
      TidyTag_WBR,      /**< WBR */
      TidyTag_XMP,      /**< XMP */
      TidyTag_NEXTID,   /**< NEXTID */

      TidyTag_ARTICLE,
      TidyTag_ASIDE,
      TidyTag_AUDIO,
      TidyTag_CANVAS,
      TidyTag_COMMAND,
      TidyTag_DATALIST,
      TidyTag_DETAILS,
      TidyTag_FIGCAPTION,
      TidyTag_FIGURE,
      TidyTag_FOOTER,
      TidyTag_HEADER,
      TidyTag_HGROUP,
      TidyTag_MARK,
      TidyTag_METER,
      TidyTag_NAV,
      TidyTag_OUTPUT,
      TidyTag_PROGRESS,
      TidyTag_SECTION,
      TidyTag_SOURCE,
      TidyTag_SUMMARY,
      TidyTag_TIME,
      TidyTag_TRACK,
      TidyTag_VIDEO,

      N_TIDY_TAGS       /**< Must be last */
    }

/* Attribute interrogation
*/

    //  Known HTML attributes
    public enum TidyAttrId
    {
      TidyAttr_UNKNOWN,           /**< UNKNOWN= */
      TidyAttr_ABBR,              /**< ABBR= */
      TidyAttr_ACCEPT,            /**< ACCEPT= */
      TidyAttr_ACCEPT_CHARSET,    /**< ACCEPT_CHARSET= */
      TidyAttr_ACCESSKEY,         /**< ACCESSKEY= */
      TidyAttr_ACTION,            /**< ACTION= */
      TidyAttr_ADD_DATE,          /**< ADD_DATE= */
      TidyAttr_ALIGN,             /**< ALIGN= */
      TidyAttr_ALINK,             /**< ALINK= */
      TidyAttr_ALT,               /**< ALT= */
      TidyAttr_ARCHIVE,           /**< ARCHIVE= */
      TidyAttr_AXIS,              /**< AXIS= */
      TidyAttr_BACKGROUND,        /**< BACKGROUND= */
      TidyAttr_BGCOLOR,           /**< BGCOLOR= */
      TidyAttr_BGPROPERTIES,      /**< BGPROPERTIES= */
      TidyAttr_BORDER,            /**< BORDER= */
      TidyAttr_BORDERCOLOR,       /**< BORDERCOLOR= */
      TidyAttr_BOTTOMMARGIN,      /**< BOTTOMMARGIN= */
      TidyAttr_CELLPADDING,       /**< CELLPADDING= */
      TidyAttr_CELLSPACING,       /**< CELLSPACING= */
      TidyAttr_CHAR,              /**< CHAR= */
      TidyAttr_CHAROFF,           /**< CHAROFF= */
      TidyAttr_CHARSET,           /**< CHARSET= */
      TidyAttr_CHECKED,           /**< CHECKED= */
      TidyAttr_CITE,              /**< CITE= */
      TidyAttr_CLASS,             /**< CLASS= */
      TidyAttr_CLASSID,           /**< CLASSID= */
      TidyAttr_CLEAR,             /**< CLEAR= */
      TidyAttr_CODE,              /**< CODE= */
      TidyAttr_CODEBASE,          /**< CODEBASE= */
      TidyAttr_CODETYPE,          /**< CODETYPE= */
      TidyAttr_COLOR,             /**< COLOR= */
      TidyAttr_COLS,              /**< COLS= */
      TidyAttr_COLSPAN,           /**< COLSPAN= */
      TidyAttr_COMPACT,           /**< COMPACT= */
      TidyAttr_CONTENT,           /**< CONTENT= */
      TidyAttr_COORDS,            /**< COORDS= */
      TidyAttr_DATA,              /**< DATA= */
      TidyAttr_DATAFLD,           /**< DATAFLD= */
      TidyAttr_DATAFORMATAS,      /**< DATAFORMATAS= */
      TidyAttr_DATAPAGESIZE,      /**< DATAPAGESIZE= */
      TidyAttr_DATASRC,           /**< DATASRC= */
      TidyAttr_DATETIME,          /**< DATETIME= */
      TidyAttr_DECLARE,           /**< DECLARE= */
      TidyAttr_DEFER,             /**< DEFER= */
      TidyAttr_DIR,               /**< DIR= */
      TidyAttr_DISABLED,          /**< DISABLED= */
      TidyAttr_ENCODING,          /**< ENCODING= */
      TidyAttr_ENCTYPE,           /**< ENCTYPE= */
      TidyAttr_FACE,              /**< FACE= */
      TidyAttr_FOR,               /**< FOR= */
      TidyAttr_FRAME,             /**< FRAME= */
      TidyAttr_FRAMEBORDER,       /**< FRAMEBORDER= */
      TidyAttr_FRAMESPACING,      /**< FRAMESPACING= */
      TidyAttr_GRIDX,             /**< GRIDX= */
      TidyAttr_GRIDY,             /**< GRIDY= */
      TidyAttr_HEADERS,           /**< HEADERS= */
      TidyAttr_HEIGHT,            /**< HEIGHT= */
      TidyAttr_HREF,              /**< HREF= */
      TidyAttr_HREFLANG,          /**< HREFLANG= */
      TidyAttr_HSPACE,            /**< HSPACE= */
      TidyAttr_HTTP_EQUIV,        /**< HTTP_EQUIV= */
      TidyAttr_ID,                /**< ID= */
      TidyAttr_ISMAP,             /**< ISMAP= */
      TidyAttr_ITEMPROP,          /**< ITEMPROP= */
      TidyAttr_LABEL,             /**< LABEL= */
      TidyAttr_LANG,              /**< LANG= */
      TidyAttr_LANGUAGE,          /**< LANGUAGE= */
      TidyAttr_LAST_MODIFIED,     /**< LAST_MODIFIED= */
      TidyAttr_LAST_VISIT,        /**< LAST_VISIT= */
      TidyAttr_LEFTMARGIN,        /**< LEFTMARGIN= */
      TidyAttr_LINK,              /**< LINK= */
      TidyAttr_LONGDESC,          /**< LONGDESC= */
      TidyAttr_LOWSRC,            /**< LOWSRC= */
      TidyAttr_MARGINHEIGHT,      /**< MARGINHEIGHT= */
      TidyAttr_MARGINWIDTH,       /**< MARGINWIDTH= */
      TidyAttr_MAXLENGTH,         /**< MAXLENGTH= */
      TidyAttr_MEDIA,             /**< MEDIA= */
      TidyAttr_METHOD,            /**< METHOD= */
      TidyAttr_MULTIPLE,          /**< MULTIPLE= */
      TidyAttr_NAME,              /**< NAME= */
      TidyAttr_NOHREF,            /**< NOHREF= */
      TidyAttr_NORESIZE,          /**< NORESIZE= */
      TidyAttr_NOSHADE,           /**< NOSHADE= */
      TidyAttr_NOWRAP,            /**< NOWRAP= */
      TidyAttr_OBJECT,            /**< OBJECT= */
      TidyAttr_OnAFTERUPDATE,     /**< OnAFTERUPDATE= */
      TidyAttr_OnBEFOREUNLOAD,    /**< OnBEFOREUNLOAD= */
      TidyAttr_OnBEFOREUPDATE,    /**< OnBEFOREUPDATE= */
      TidyAttr_OnBLUR,            /**< OnBLUR= */
      TidyAttr_OnCHANGE,          /**< OnCHANGE= */
      TidyAttr_OnCLICK,           /**< OnCLICK= */
      TidyAttr_OnDATAAVAILABLE,   /**< OnDATAAVAILABLE= */
      TidyAttr_OnDATASETCHANGED,  /**< OnDATASETCHANGED= */
      TidyAttr_OnDATASETCOMPLETE, /**< OnDATASETCOMPLETE= */
      TidyAttr_OnDBLCLICK,        /**< OnDBLCLICK= */
      TidyAttr_OnERRORUPDATE,     /**< OnERRORUPDATE= */
      TidyAttr_OnFOCUS,           /**< OnFOCUS= */
      TidyAttr_OnKEYDOWN,         /**< OnKEYDOWN= */
      TidyAttr_OnKEYPRESS,        /**< OnKEYPRESS= */
      TidyAttr_OnKEYUP,           /**< OnKEYUP= */
      TidyAttr_OnLOAD,            /**< OnLOAD= */
      TidyAttr_OnMOUSEDOWN,       /**< OnMOUSEDOWN= */
      TidyAttr_OnMOUSEMOVE,       /**< OnMOUSEMOVE= */
      TidyAttr_OnMOUSEOUT,        /**< OnMOUSEOUT= */
      TidyAttr_OnMOUSEOVER,       /**< OnMOUSEOVER= */
      TidyAttr_OnMOUSEUP,         /**< OnMOUSEUP= */
      TidyAttr_OnRESET,           /**< OnRESET= */
      TidyAttr_OnROWENTER,        /**< OnROWENTER= */
      TidyAttr_OnROWEXIT,         /**< OnROWEXIT= */
      TidyAttr_OnSELECT,          /**< OnSELECT= */
      TidyAttr_OnSUBMIT,          /**< OnSUBMIT= */
      TidyAttr_OnUNLOAD,          /**< OnUNLOAD= */
      TidyAttr_PROFILE,           /**< PROFILE= */
      TidyAttr_PROMPT,            /**< PROMPT= */
      TidyAttr_RBSPAN,            /**< RBSPAN= */
      TidyAttr_READONLY,          /**< READONLY= */
      TidyAttr_REL,               /**< REL= */
      TidyAttr_REV,               /**< REV= */
      TidyAttr_RIGHTMARGIN,       /**< RIGHTMARGIN= */
      TidyAttr_ROWS,              /**< ROWS= */
      TidyAttr_ROWSPAN,           /**< ROWSPAN= */
      TidyAttr_RULES,             /**< RULES= */
      TidyAttr_SCHEME,            /**< SCHEME= */
      TidyAttr_SCOPE,             /**< SCOPE= */
      TidyAttr_SCROLLING,         /**< SCROLLING= */
      TidyAttr_SELECTED,          /**< SELECTED= */
      TidyAttr_SHAPE,             /**< SHAPE= */
      TidyAttr_SHOWGRID,          /**< SHOWGRID= */
      TidyAttr_SHOWGRIDX,         /**< SHOWGRIDX= */
      TidyAttr_SHOWGRIDY,         /**< SHOWGRIDY= */
      TidyAttr_SIZE,              /**< SIZE= */
      TidyAttr_SPAN,              /**< SPAN= */
      TidyAttr_SRC,               /**< SRC= */
      TidyAttr_STANDBY,           /**< STANDBY= */
      TidyAttr_START,             /**< START= */
      TidyAttr_STYLE,             /**< STYLE= */
      TidyAttr_SUMMARY,           /**< SUMMARY= */
      TidyAttr_TABINDEX,          /**< TABINDEX= */
      TidyAttr_TARGET,            /**< TARGET= */
      TidyAttr_TEXT,              /**< TEXT= */
      TidyAttr_TITLE,             /**< TITLE= */
      TidyAttr_TOPMARGIN,         /**< TOPMARGIN= */
      TidyAttr_TYPE,              /**< TYPE= */
      TidyAttr_USEMAP,            /**< USEMAP= */
      TidyAttr_VALIGN,            /**< VALIGN= */
      TidyAttr_VALUE,             /**< VALUE= */
      TidyAttr_VALUETYPE,         /**< VALUETYPE= */
      TidyAttr_VERSION,           /**< VERSION= */
      TidyAttr_VLINK,             /**< VLINK= */
      TidyAttr_VSPACE,            /**< VSPACE= */
      TidyAttr_WIDTH,             /**< WIDTH= */
      TidyAttr_WRAP,              /**< WRAP= */
      TidyAttr_XML_LANG,          /**< XML_LANG= */
      TidyAttr_XML_SPACE,         /**< XML_SPACE= */
      TidyAttr_XMLNS,             /**< XMLNS= */

      TidyAttr_EVENT,             /**< EVENT= */
      TidyAttr_METHODS,           /**< METHODS= */
      TidyAttr_N,                 /**< N= */
      TidyAttr_SDAFORM,           /**< SDAFORM= */
      TidyAttr_SDAPREF,           /**< SDAPREF= */
      TidyAttr_SDASUFF,           /**< SDASUFF= */
      TidyAttr_URN,               /**< URN= */

      TidyAttr_ASYNC,
      TidyAttr_AUTOCOMPLETE,
      TidyAttr_AUTOFOCUS,
      TidyAttr_AUTOPLAY,
      TidyAttr_CHALLENGE,
      TidyAttr_CONTENTEDITABLE,
      TidyAttr_CONTEXTMENU,
      TidyAttr_CONTROLS,
      TidyAttr_DEFAULT,
      TidyAttr_DIRNAME,
      TidyAttr_DRAGGABLE,
      TidyAttr_DROPZONE,
      TidyAttr_FORM,
      TidyAttr_FORMACTION,
      TidyAttr_FORMENCTYPE,
      TidyAttr_FORMMETHOD,
      TidyAttr_FORMNOVALIDATE,
      TidyAttr_FORMTARGET,
      TidyAttr_HIDDEN,
      TidyAttr_HIGH,
      TidyAttr_ICON,
      TidyAttr_KEYTYPE,
      TidyAttr_KIND,
      TidyAttr_LIST,
      TidyAttr_LOOP,
      TidyAttr_LOW,
      TidyAttr_MANIFEST,
      TidyAttr_MAX,
      TidyAttr_MEDIAGROUP,
      TidyAttr_MIN,
      TidyAttr_NOVALIDATE,
      TidyAttr_OPEN,
      TidyAttr_OPTIMUM,
      TidyAttr_OnABORT,
      TidyAttr_OnAFTERPRINT,
      TidyAttr_OnBEFOREPRINT,
      TidyAttr_OnCANPLAY,
      TidyAttr_OnCANPLAYTHROUGH,
      TidyAttr_OnCONTEXTMENU,
      TidyAttr_OnCUECHANGE,
      TidyAttr_OnDRAG,
      TidyAttr_OnDRAGEND,
      TidyAttr_OnDRAGENTER,
      TidyAttr_OnDRAGLEAVE,
      TidyAttr_OnDRAGOVER,
      TidyAttr_OnDRAGSTART,
      TidyAttr_OnDROP,
      TidyAttr_OnDURATIONCHANGE,
      TidyAttr_OnEMPTIED,
      TidyAttr_OnENDED,
      TidyAttr_OnERROR,
      TidyAttr_OnHASHCHANGE,
      TidyAttr_OnINPUT,
      TidyAttr_OnINVALID,
      TidyAttr_OnLOADEDDATA,
      TidyAttr_OnLOADEDMETADATA,
      TidyAttr_OnLOADSTART,
      TidyAttr_OnMESSAGE,
      TidyAttr_OnMOUSEWHEEL,
      TidyAttr_OnOFFLINE,
      TidyAttr_OnONLINE,
      TidyAttr_OnPAGEHIDE,
      TidyAttr_OnPAGESHOW,
      TidyAttr_OnPAUSE,
      TidyAttr_OnPLAY,
      TidyAttr_OnPLAYING,
      TidyAttr_OnPOPSTATE,
      TidyAttr_OnPROGRESS,
      TidyAttr_OnRATECHANGE,
      TidyAttr_OnREADYSTATECHANGE,
      TidyAttr_OnREDO,
      TidyAttr_OnRESIZE,
      TidyAttr_OnSCROLL,
      TidyAttr_OnSEEKED,
      TidyAttr_OnSEEKING,
      TidyAttr_OnSHOW,
      TidyAttr_OnSTALLED,
      TidyAttr_OnSTORAGE,
      TidyAttr_OnSUSPEND,
      TidyAttr_OnTIMEUPDATE,
      TidyAttr_OnUNDO,
      TidyAttr_OnVOLUMECHANGE,
      TidyAttr_OnWAITING,
      TidyAttr_PATTERN,
      TidyAttr_PLACEHOLDER,
      TidyAttr_POSTER,
      TidyAttr_PRELOAD,
      TidyAttr_PUBDATE,
      TidyAttr_RADIOGROUP,
      TidyAttr_REQUIRED,
      TidyAttr_REVERSED,
      TidyAttr_SANDBOX,
      TidyAttr_SCOPED,
      TidyAttr_SEAMLESS,
      TidyAttr_SIZES,
      TidyAttr_SPELLCHECK,
      TidyAttr_SRCDOC,
      TidyAttr_SRCLANG,
      TidyAttr_STEP,


      N_TIDY_ATTRIBS              /**< Must be last */
    }

}
