﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.IO;

namespace TidyCmd
{
    /// <summary>
    /// Provide variouse Help messages
    /// 
    /// Modified by Dennis Lang - 2014
    /// http://home.comcast.net/~lang.dennis/
    /// 
    /// ========== License ==========
    /// This file is part of TidyCmd.
    ///
    /// TidyCmd is free software: you can redistribute it and/or modify
    /// it under the terms of the GNU General Public License as published by
    /// the Free Software Foundation, either version 3 of the License, or
    /// (at your option) any later version.
    ///
    /// TidyCmd is distributed in the hope that it will be useful,
    /// but WITHOUT ANY WARRANTY; without even the implied warranty of
    /// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    /// GNU General Public License for more details.
    ///
    /// The GNU General Public License: <http://www.gnu.org/licenses/>.
    /// </summary>
    class TidyHelp
    {
        static string VersionStr()
        {
            // DateTime buildDateTime = GetBuildDateTime();
            Assembly assembly = Assembly.GetExecutingAssembly();
            Version version = assembly.GetName().Version;
            return "TidyCmd %0e" + string.Format("v{0}.{1}", version.Major, version.Minor) + "%0b Dennis Lang 2014%0f"
                 + " http://home.comcast.net/~lang.dennis/\n";
        }

        const string sDescription = ""
            + "HTML parser and pretty printer derived from Dave Raggett's Tidy\n"
            + "\n";

        const string sAuthors = ""
            + "%0eContributing Author(s):%0f \n"
            + "\n"
            + "  %0bDave Raggett%0f <dsr@w3.org>\n"
            + "  %0bAndy Quick%0f <ac.quick@sympatico.ca> (translation to Java)\n"
            + "  %0bDennis Lang%0f <landenlabs@gmail.com> 2014 Added command line frontend \n"
            + "\n"
            + "The contributing author(s) would like to thank all those who\n"
            + "helped with testing, bug fixes, and patience.  This wouldn't\n"
            + "have been possible without all of you.\n";

        const string sCopyright = ""
            + "%0eCOPYRIGHT NOTICE (For TidyCmd):%0f\n"
            + "  Copyright (c) 2014 Dennis Lang and LanDen Labs \n"
            + "\n"
            + "%0eCOPYRIGHT NOTICE (For library TidyNetLib):%0f\n"
            + "  Copyright (c) 1998-2000 World Wide Web Consortium (Massachusetts\n"
            + "  Institute of Technology, Institut National de Recherche en\n"
            + "  Informatique et en Automatique, Keio University). All Rights\n"
            + "  Reserved.\n"
            + "\n"
            + "  This software and documentation is provided \"as is,\" and\n"
            + "  the copyright holders and contributing author(s) make no\n"
            + "  representations or warranties, express or implied, including\n"
            + "  but not limited to, warranties of merchantability or fitness\n"
            + "  for any particular purpose or that the use of the software or\n"
            + "  documentation will not infringe any third party patents,\n"
            + "  copyrights, trademarks or other rights. \n"
            + "\n"
            + "  The copyright holders and contributing author(s) will not be\n"
            + "  liable for any direct, indirect, special or consequential damages\n"
            + "  arising out of any use of the software or documentation, even if\n"
            + "  advised of the possibility of such damage.\n"
            + "\n"
            + "  Permission is hereby granted to use, copy, modify, and distribute\n"
            + "  this source code, or portions hereof, documentation and executables,\n"
            + "  for any purpose, without fee, subject to the following restrictions:\n"
            + "\n"
            + "    1. The origin of this source code must not be misrepresented.\n"
            + "    2. Altered versions must be plainly marked as such and must\n"
            + "       not be misrepresented as being the original source.\n"
            + "    3. This Copyright notice may not be removed or altered from any\n"
            + "       source or altered source distribution.\n"
            + "\n"
            + "  The copyright holders and contributing author(s) specifically\n"
            + "  permit, without fee, and encourage the use of this source code\n"
            + "  as a component for supporting the Hypertext Markup Language in\n"
            + "  commercial products. If you use this source code in a product,\n"
            + "  acknowledgment is not required but would be appreciated.\n";

        const string sHelpCmd = "\n"
            + "TidyCmd [options] infile1 [infile2]...\n\n"
            + "\n"
            + "Options for HTML TidyCmd:\n"
            + "\n"
            + "%0eFile manipulation%0f\n"
            + "-----------------\n"
            + " -inFile=<%0bfile%0f>      Specify input file or web link\n"
            + " -outFile=<%0bfile%0f>     Write output to the specified <%0bfile%0f>\n"
            + " -modify             Modify the original input files\n"
            + "\n"
            + " -config=<%0bfile%0f>      Set configuration options from the specified <%0bfile%0f>\n"
            + "\n"
            + " -errFile=<%0bfile%0f>     Write errors and warnings to the specified <%0bfile%0f>\n"
            + " -logFile=<%0bfile%0f>     Write internal log message to specified <%0bfile%0f>\n"
            + "\n"
            + "%0eProcessing directives%0f\n"
            + "---------------------\n"
            + " -indent             Indent element content\n"
            + " -wrap=<%0bcolumn%0f>      Wrap text at the specified <%0bcolumn%0f>.\n"
            + " -upper              Force tags to upper case\n"
            + " -clean              Replace FONT, NOBR and CENTER tags by CSS\n"
            + " -gdoc               Produce clean version of html exported by google docs\n"
            + " -numeric            Output numeric rather than named entities\n"
            + " -omit               Omit optional start tags and end tags\n"
            + "\n"
            + " -warnings           Show warnings \n"
            + " -errors             Show errors \n"
            + " -resources          Show resources (img, href, url) \n"
            + " -quiet              Suppress nonessential output \n"
            + " -progress           When using -errFile, use -progress to show page progress \n"
            + " -testurls=all       Test ability to load image and href urls \n"
            + "\n"
            + " -loadhrefs=host     Load and follow HREF urls, use to validate entire website \n"
            + "\n"
            + " -xml                Specify the input is well formed XML\n"
            + " -asxml, -asxhtml    Convert HTML to well formed XHTML\n"
            + " -ashtml             Force XHTML to well formed HTML\n"
      
            + "\n"
            + "%0eCharacter encodings%0f\n"
            + "-------------------\n"
            + " -raw                Output values above 127 without conversion to entities\n"
            + " -ascii              Use ISO-8859-1 for input, US-ASCII for output\n"
            + " -latin0             Use ISO-8859-15 for input, US-ASCII for output\n"
            + " -latin1             Use ISO-8859-1 for both input and output\n"
            + " -utf8               Use UTF-8 for both input and output\n"
            + "\n"
            + "%0eMiscellaneous%0f\n"
            + "-------------\n"
            + " -version, -v        Show the version of Tidy\n"
            + " -?,-help,-helpfull  List the command line options\n"
            + " -help-xml           List the command line options in XML format\n"
            + " -help-config        List all configuration options\n"
            + " -xml-config         List all configuration options in XML format\n"
            + " -show-config        List the current configuration settings\n"
            + "\n"
            + "%0eConfiguration Options%0f\n"
            + "---------------------\n"
            + " --<%0coptionName%0f> <%0boptionValue%0f> \n"
            + "Use -- to proceed configuration option overrides\n"
            + "See -config, -help-config and -show-config\n"
            + "\n"
            + "Input/Output default to stdin/stdout respectively.\n"
            + "\n";

        const string sHelpWeb = ""
            + "For more information on this HTML5-aware experimental fork of Tidy,\n"
            + "see http://w3c.github.com/tidy-html5/\n"
            + "\n"
            + "For more information on HTML, see the following:\n"
            + "\n"
            + "  HTML: Edition for Web Authors (the latest HTML specification)\n"
            + "  http://dev.w3.org/html5/spec-author-view\n"
            + "\n"
            + "  HTML: The Markup Language (an HTML language reference)\n"
            + "  http://dev.w3.org/html5/markup/\n"
            + "\n"
            + "File bug reports at https://github.com/w3c/tidy-html5/issues/\n"
            + "or send questions and comments to html-tidy@w3.org\n"
            + "\n"
            + "Validate your HTML documents using the W3C Nu Markup Validator:\n"
            + "\n"
            + "  http://validator.w3.org/nu/\n"
            + "\n";



        public enum HelpLevel { eHelpFull, eHelpVersion, eHelpCmd, eHelpWeb, eHelpXml, eHelpCfg }
        public static void ShowHelp(HelpLevel helpLevel)
        {
            switch (helpLevel)
            {
                case HelpLevel.eHelpFull:
                    ConWriter.ColorizeLine(VersionStr() + sDescription + sAuthors + sCopyright + sHelpWeb);
                    ConWriter.ColorizeLine(sHelpCmd);
                    break;
                case HelpLevel.eHelpVersion:
                    ConWriter.ColorizeLine(VersionStr());
                    break;
                case HelpLevel.eHelpCmd:
                    ConWriter.ColorizeLine(VersionStr() + sHelpCmd);
                    break;
                case HelpLevel.eHelpWeb:
                    ConWriter.ColorizeLine(sHelpWeb);
                    break;
                case HelpLevel.eHelpXml:
                    break;
                case HelpLevel.eHelpCfg:
                    ShowReferenceHtml();
                    break;
            }
            Console.WriteLine("");
        }

        public static void ShowReferenceHtml()
        {
#if true
            Assembly assembly = Assembly.GetExecutingAssembly();
            TextReader htmlReader = new StreamReader(assembly.GetManifestResourceStream("TidyCmd.tidy-ref.html"));
            string tmpOutFile = ".\tidy-ref.html";
            TextWriter outWriter = new StreamWriter(tmpOutFile);
            outWriter.Write(htmlReader.ReadToEnd());
            outWriter.Close();
            System.Diagnostics.Process.Start(tmpOutFile);
#else
            //
            // Attempt to convert HTML Tidy documentation to a Text stream
            // to avoid having two copies of documentation. Output was not
            // nice enough, so I gave up.
            //
            Assembly assembly = Assembly.GetExecutingAssembly();
            TextReader htmlReader = new StreamReader(assembly.GetManifestResourceStream("TidyCmd.tidy-ref.html"));
            TextReader xsltReader = new StreamReader(assembly.GetManifestResourceStream("TidyCmd.markdown.xslt"));
            var html = XDocument.Load(htmlReader);

            var transformer = new XslCompiledTransform();
            // var xsl = XDocument.Parse(@"<?xml version='1.0'?><xsl:stylesheet version=""1.0"" xmlns:xsl=""http://www.w3.org/1999/XSL/Transform"" exclude-result-prefixes=""xsl""><xsl:output method=""html"" indent=""yes"" version=""4.0"" omit-xml-declaration=""yes"" encoding=""UTF-8"" /><xsl:template match=""/""><xsl:value-of select=""."" /></xsl:template></xsl:stylesheet>");
            var xsl = XDocument.Load(xsltReader);
            // xsl.CreateEntityReference("nbsp");
            transformer.Load(xsl.CreateNavigator());
            transformer.Transform(html.CreateReader(), null, Console.Out);
#endif
        }
    }
}
