using System;
using System.Collections;
using System.Collections.Generic;
using TidyNetLib.tidy;

namespace TidyNetLib
{
	/// <summary>
	/// Collection of TidyMessages
	/// </summary>
	[Serializable]
	public class TidyMessageCollection : CollectionBase
	{
		/// <summary>
		/// Public default constructor
		/// </summary>
		public TidyMessageCollection()
		{
		}

		/// <summary>
		/// Adds a message.
		/// </summary>
		/// <param name="message">The message to add.</param>
		public void Add(TidyMessage message)
		{
            if (!InnerList.Contains(message))
            {
                if (message.Level == MessageLevel.Error)
                    Errors++;
                else if (message.Level == MessageLevel.Warning)
                    Warnings++;

                InnerList.Add(message);
            }
		}

		/// <summary> Errors - the number of errors that occurred in the most
		/// recent parse operation
		/// </summary>
        public virtual int Errors
        {
            get;
            private set;
        }
		
		/// <summary> Warnings - the number of warnings that occurred in the most
		/// recent parse operation
		/// </summary>
        public virtual int Warnings
        {
            get;
            private set;
        }

        void Clear()
        {
            base.Clear();
            Errors = Warnings = 0;
        }
	}
}
